@@ -77,7 +77,7 @@ int main( int argc, char *argv[] )
   shown_fields = g_list_append(shown_fields, "Third field");
   shown_fields = g_list_append(shown_fields, "Fourth field");
 
-  print = e_contact_print_dialog_new(NULL, NULL);
+  print = e_contact_print_dialog_new(NULL, NULL, NULL);
   gtk_widget_show_all(print);
   g_signal_connect(print, "close", G_CALLBACK(test_close), NULL);
 
