@@ -27,6 +27,8 @@
 #include <sys/types.h>
 #include <glib-object.h>
 #include <limits.h>
+#include <gconf/gconf-client.h>
+#include <cairo.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -154,6 +156,8 @@ char     *e_read_file                                                      (cons
 int       e_write_file                                                     (const char        *filename,
 									    const char        *data,
 									    int                flags);
+int       e_write_file_uri                                                 (const char        *filename,
+									    const char        *data);
 int       e_write_file_mkstemp                                             (char              *filename,
 									    const char        *data);
 
@@ -211,6 +215,10 @@ gchar    *e_ascii_dtostr                                                   (gcha
    Less than 0 for the int means copy the whole string. */
 gchar    *e_strdup_append_strings                                          (gchar             *first_string,
 									    ...);
+cairo_font_options_t * get_font_options ();
+
+void e_file_update_save_path(char *uri, gboolean free);
+char *e_file_get_save_path(void);
 
 #ifdef __cplusplus
 }
