@@ -172,6 +172,7 @@ static const char *authors[] = {
 	"Arik Devens",
 	"Arturo Espinosa Aldama",
 	"Arunprakash",
+	"Arvind Sundararajan",
 	"Bastien Nocera",
 	"Benjamin Kahn",
 	"Benjamin Berg",
@@ -281,6 +282,7 @@ static const char *authors[] = {
 	"Herbert V. Riedel",
 	"H P Nadig",
 	"H\xC3\xA9" "ctor Garc\xC3\xAD" "a \xC3\x81" "lvarez",
+	"Hiroyuki Ikezoe",
 	"Iain Holmes",
 	"Ian Campbell",
 	"Ilkka Tuohela",
@@ -371,6 +373,7 @@ static const char *authors[] = {
 	"Matias Mutchinick",
 	"Matt Bissiri",
 	"Matt Brown",
+	"Matthew Barnes",
 	"Matthew Hall",
 	"Matthew Loper",
 	"Matthew Wilson",
@@ -540,6 +543,7 @@ static const char *documentors[] = {
 	"David Trowbridge",
 	"Jessica Prabhakar",
 	"Binika Preet",
+	"Radhika Nair",
 	NULL
 };
 
